@@ -11,10 +11,12 @@
 #' @param x a character object, a \pkg{quanteda} corpus, or a TIF-compliant
 #'   corpus data.frame (see \url{https://github.com/ropensci/tif})
 #' @param pos logical whether to return universal dependency POS tagset
-#'   (http://universaldependencies.org/u/pos/all.html)
+#'   \url{http://universaldependencies.org/u/pos/})
 #' @param tag logical whether to return detailed part-of-speech tags, for the
 #'   langage model \code{en}, it uses the OntoNotes 5 version of the Penn
-#'   Treebank tag set (https://spacy.io/docs/usage/pos-tagging#pos-schemes)
+#'   Treebank tag set (\url{https://spacy.io/docs/usage/pos-tagging#pos-schemes}). 
+#' Annotation specifications for other available languages are available on the 
+#' spaCy website (\url{https://spacy.io/api/annotation}).
 #' @param lemma logical; inlucde lemmatized tokens in the output (lemmatization 
 #'   may not work properly for non-English models)
 #' @param entity logical; if \code{TRUE}, report named entities
@@ -65,6 +67,13 @@ spacy_parse.character <- function(x,
         stop("Document parsing failed")
     }
     
+    ## check the omit_entity status
+    if (entity == TRUE & getOption("spacy_entity") == FALSE) {
+        message("entity == TRUE is ignored because spaCy model is initialized without Entity Recognizer")
+        message("In order to turn on entity recognition, run spacy_finalize(); spacy_initialize(entity = TURE)")
+        entity <- FALSE
+    }
+    
     tokens <- get_tokens(spacy_out)
     ntokens <- get_ntokens(spacy_out)
     ntokens_by_sent <- get_ntokens_by_sent(spacy_out)
